# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and 
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

##########################################################
# micala_db_backup.py
#
# Author: Jeremy Herr (herrj@umich.edu)
#
# This script calls mysqldump to back up the micala monitoring database
##########################################################

import glob
import time
import conf
from micala import *

timestamp = time.localtime()

fileBackup = "micala_backup_%04d%02d%02d.sql" % \
    (timestamp.tm_year, timestamp.tm_mon, timestamp.tm_mday)

pathBackup = os.path.join(conf.dirDBBackup, fileBackup)

command = "%s --add-drop-table" \
    " --user=%s" \
    " --password=%s" \
    " micala" \
    " Lectures" \
    " Machines" \
    " Status" \
    " Tasks" \
    " Users " \
    " Venues" \
    " > \"%s\"" % \
    (conf.mysqldump, conf.DBAdminUser, conf.DBAdminPW, pathBackup)

# execute mysqldump command
if os.system(command) != 0:
    # TODO: notify monitoring database or send email that backup failed
    pass

# delete old backup files.
# First get a list of all .sql files in dirDBBackup
listBackups = glob.glob(os.path.join(conf.dirDBBackup, '*.sql'))

# check stats of each file and delete old files
for fileBackup in listBackups:
    pathBackup = os.path.join(conf.dirDBBackup, fileBackup) 
    if os.path.exists(pathBackup):
        # Get file info
        statFile = os.stat(pathBackup)
        # Calculate how many seconds old the file is
        ageFileSeconds = time.mktime(time.localtime()) - statFile.st_ctime
        ageFileDays    = ageFileSeconds / 86400
        print "age of file is: %f days" % ageFileDays
        if ageFileDays > conf.ageBackupsToKeep:
            print "deleting database backup %s because it is %f days old, which is older than %d days" % (fileBackup, ageFileDays, conf.ageBackupsToKeep)
            os.remove(pathBackup)
