# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import *

import re

class MicalaLectureDetailsController(MicalaController):
    "Lecture Details Controller to view, crop and start processing the lecture object"
    # template file for lecture details controller
    templatePage = "lectureDetails"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getLectureAndStatusLists(self, idLecture):
        "Retrieve lecture details and show it"

        if MicalaCommunication.lectureExists(idLecture):
            (lecture, listStatuses) = MicalaCommunication.getLectureAndStatusLists(idLecture)
            listTasks = MicalaCommunication.getTaskList()
            (flagFinished, dictLectureLatestStatus) = Common.getLectureLatestStatus(lecture["LOID"], listTasks, listStatuses)

            self.pageVars['dictLectureLatestStatus'] = dictLectureLatestStatus

            self.pageVars['idLecture'] = idLecture
            self.pageVars['lectureObjectID'] = lecture["LOID"]
            self.pageVars['lecture'] = lecture
            self.pageVars['lectureRoom'] = dictLectureLatestStatus["recording"]["room"]
            self.pageVars['lectureMachine'] = dictLectureLatestStatus["recording"]["machine"]
            self.pageVars['lectureConferenceID'] = Common.getConfId(lecture["IndicoID"])
            self.pageVars['listStatuses'] = listStatuses
        else:
            # display an error if parameters has been modified
            self.pageVars['error'] = 'Lecture ID not found in database.'

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()
        # default values for variables
        self.pageVars['error'] = None
        self.pageVars['idLecture'] = None
        self.pageVars['lectureObjectID'] = None
        self.pageVars['lecture'] = None
        self.pageVars['lectureRoom'] = None
        self.pageVars['lectureMachine'] = None
        self.pageVars['lectureConferenceID'] = None
        self.pageVars['listStatuses'] = None

        if params is not None and params.has_key("idLecture"):
            # get all tasks and their status
            if params["idLecture"].value.isdigit():
                self._getLectureAndStatusLists(params["idLecture"].value)
            else:
                # display an error if parameters has been modified
                self.pageVars['error'] = 'Lecture ID not found in database.'

