# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import *

class MicalaLectureStartProcessingController(MicalaController):
    "Start the processing task for the lecture passed"
    # template file for lecture details controller
    templatePage = "lectureStartProcessing"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _startProcessing(self, params, idLecture):
        "Start the processing task and insert crop times into the database"
        self.pageVars['messages'] = ''

        if MicalaCommunication.lectureExists(idLecture):
            # get the machine name
            idMachine = MicalaCommunication.getIdMachine(conf.curr_machine)
            for task in conf.tasksWebLecture:
                # get the task id
                idTask = MicalaCommunication.getIdTask(task)
                # request processing to daemon
                MicalaCommunication.reportStatus("REQUESTED", "", idMachine, idTask, idLecture)
                self.pageVars['messages'] += "Request submitted for task %s<br>" % task

            # check if crop is needed
            cropBegin = None
            if params.has_key('cropBegin') and params['cropBegin'].value is not None and params['cropBegin'].value.isdigit():
                cropBegin = params['cropBegin'].value
            cropEnd = None
            if params.has_key('cropEnd') and params['cropEnd'].value is not None and params['cropEnd'].value.isdigit():
                cropEnd = params['cropEnd'].value

            if cropBegin is not None or cropEnd is not None:
                # insert crop value
                MicalaCommunication.updateLectureCropTimes(idLecture, cropBegin, cropEnd)
                self.pageVars['messages'] +=  "Crop time correctly set. The video will be cropped."
            else:
                self.pageVars['messages'] +=  "No cropping requested. The video will not be cropped."
        else:
            # display an error if parameters has been modified
            self.pageVars['error'] = 'Lecture ID not found in database.'

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()
        # default values for variables
        self.pageVars['error'] = ''

        if params is not None:
            if params.has_key('idLecture'):
                idLecture = params['idLecture'].value
                if idLecture.isdigit():
                    # start processing
                    self._startProcessing(params, idLecture)
                else:
                    # display an error if parameters has been modified
                    self.pageVars['error'] = 'Lecture ID not found in database.'
