# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import *

class MicalaLectureTasksStatusController(MicalaController):
    "Lecture Tasks status Controller to show the status of each task"
    # template file for lecture details controller
    templatePage = "lectureTasksStatus"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getLectureTasksStatusList(self, idLecture):
        "Retrieve lecture Status list and show it"
        self.pageVars['idLecture'] = idLecture

        if MicalaCommunication.lectureExists(idLecture):
            (lecture, listStatuses) = MicalaCommunication.getLectureAndStatusLists(idLecture)
            listTasks = MicalaCommunication.getTaskList()
            (flagFinished, dictLectureLatestStatus) = Common.getLectureLatestStatus(lecture["LOID"], listTasks, listStatuses)

            self.pageVars['dictLectureLatestStatus'] = dictLectureLatestStatus
            self.pageVars['listTasks'] = listTasks
        else:
            # display an error if parameters has been modified
            self.pageVars['error'] = 'Lecture ID not found in database.'

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()
        # default values for variables
        self.pageVars['error'] = ''
        self.pageVars['idLecture'] = ''
        self.pageVars['dictLectureLatestStatus'] = None
        self.pageVars['listTasks'] = None

        if params is not None and params.has_key("idLecture"):
            # get tasks status list
            if params["idLecture"].value.isdigit():
                self._getLectureTasksStatusList(params["idLecture"].value)
            else:
                # display an error if parameters has been modified
                self.pageVars['error'] = 'Lecture ID not found in database.'

