# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

class MicalaLectureToDoController(MicalaController):
    "Lecture to-do Controller to show to user human tasks"
    # template file for lecture details controller
    templatePage = "lectureToDo"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getLectureToDoList(self, idLecture):
        "Retrieve lecture to do list and show it"
        self.pageVars['idLecture'] = idLecture

        if MicalaCommunication.lectureExists(idLecture):
            # human tasks
            # id   name
            # 1    start recording
            # 1    stop recording
            # 2    processing
            # 6    match with recording manager
            # 8    check slides
            # N/A     create recording manager link

            # when a human task is done, increase the percentage of work done
            # 100% / 6 = 16% -> 16% * 6 = 96% (you must have 100%, so start from 4)
            percentStep = 16
            percentStart = 4
            self.pageVars['progressValue'] = percentStart;

            # for each human tasks, the status can be: miss, done, error
            # miss, done, error are also the class for the stylesheet

            # get latest status for task 'recording'
            self.pageVars['recordingStartStatus'] = 'miss'
            recordingStatus = MicalaCommunication.getLatestStatus(idLecture, "recording")
            if recordingStatus is not None:
                if recordingStatus['Status'] == 'START' or recordingStatus['Status'] == 'RUNNING' or recordingStatus['Status'] == 'COMPLETE':
                    # human task done
                    self.pageVars['recordingStartStatus'] = 'done'
                    self.pageVars['progressValue'] = percentStep + percentStart
                if recordingStatus['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['recordingStartStatus'] = 'error'

            self.pageVars['recordingStopStatus'] = 'miss'
            # use the same recordingStatus
            if recordingStatus is not None:
                if recordingStatus['Status'] == 'COMPLETE':
                    # human task done
                    self.pageVars['recordingStopStatus'] = 'done'
                    self.pageVars['progressValue'] = percentStep * 2 +percentStart
                if recordingStatus['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['recordingStopStatus'] = 'error'

            # get latest status for task 'processing'
            self.pageVars['processingStatus'] = 'miss'
            processingStatus = MicalaCommunication.getLatestStatus(idLecture, "converting video to LO")
            if processingStatus is not None:
                if processingStatus['Status'] != 'ERROR':
                    # human task done
                    self.pageVars['processingStatus'] = 'done'
                    self.pageVars['progressValue'] = percentStep * 3 + percentStart
                if processingStatus['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['processingStatus'] = 'error'

            # get latest status for task 'match with recording manager'
            self.pageVars['matchRecordingManagerStatus'] = 'miss'
            matchingRecordingManagerStatus = MicalaCommunication.getLatestStatus(idLecture, "metadata export to micala")
            if matchingRecordingManagerStatus is not None:
                if matchingRecordingManagerStatus['Status'] != 'ERROR':
                    # human task done
                    self.pageVars['matchRecordingManagerStatus'] = 'done'
                    self.pageVars['progressValue'] = percentStep * 4 + percentStart
                if matchingRecordingManagerStatus['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['matchRecordingManagerStatus'] = 'error'

            # get latest status for task 'check slides'
            self.pageVars['checkSlidesStatus'] = 'miss'
            checkSlidesStatus = MicalaCommunication.getLatestStatus(idLecture, "uploading completed LO")
            if checkSlidesStatus is not None:
                if checkSlidesStatus['Status'] != 'ERROR':
                    # human task done
                    self.pageVars['checkSlidesStatus'] = 'done'
                    self.pageVars['progressValue'] = percentStep * 5 + percentStart
                if checkSlidesStatus['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['checkSlidesStatus'] = 'error'

            # get latest status for task 'create Indico link to CDS'
            self.pageVars['indicoLinkToCDS'] = 'miss'
            indicoLinkToCDS = MicalaCommunication.getLatestStatus(idLecture, "Indico link to CDS")
            if indicoLinkToCDS is not None:
                if indicoLinkToCDS['Status'] != 'ERROR':
                    # human task done
                    self.pageVars['indicoLinkToCDS'] = 'done'
                    self.pageVars['progressValue'] = percentStep * 5 + percentStart
                if indicoLinkToCDS['Status'] == 'ERROR':
                    # error with the task
                    self.pageVars['indicoLinkToCDS'] = 'error'
        else:
            # display an error if parameters has been modified
            self.pageVars['error'] = 'Lecture ID not found in database.'

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()
        # default values for variables
        self.pageVars['error'] = None
        self.pageVars['idLecture'] = None

        if params is not None and params.has_key("idLecture"):
            # get to-do list
            if params["idLecture"].value.isdigit():
                self._getLectureToDoList(params["idLecture"].value)
            else:
                # display an error if parameters has been modified
                self.pageVars['error'] = 'Lecture ID not found in database.'

