# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

class MicalaMachineDetailsController(MicalaController):
    "Detail Machine Controller to get all details of a machine"
    # template file for machine details controller
    templatePage = "machineDetails"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getMachine(self, idMachine):
        "Get a machine"
        if idMachine == '0':
            # New machine
            machine = dict()
            machine["idMachine"] = '0'
            machine["Hostname"] = ''
            machine["IP"] = ''
            machine["CPU"] = ''
            machine["Memory"] = ''
            machine["Disk"] = ''
            machine["OS"] = ''
            machine["idVenue"] = ''
        else:
            # Edit machine
            machine = MicalaCommunication.getMachine(idMachine)

        self.pageVars['machine'] = machine

        # get all venues
        listVenues = MicalaCommunication.getVenueList()
        self.pageVars['listVenues'] = listVenues


    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()

        # check parameters
        if params is not None and params.has_key("action"):
            if params["action"].value == "getMachine" and params["idMachine"].value.isdigit():
                self._getMachine(params["idMachine"].value)

