# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

class MicalaMachinesController(MicalaController):
    "Machines Controller to manage machines"
    # template file for machines controller
    templatePage = "machines"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getMachineList(self):
        "Retrieve machineList and show it"
        listMachines = MicalaCommunication.getMachineList()
        self.pageVars['listMachines'] = listMachines

    @classmethod
    def _insertNewMachine(self, hostname, ip, cpu, memory, disk, os, idVenue):
        "Insert a new machine"
        MicalaCommunication.insertNewMachine(hostname, ip, cpu, memory, disk, os, idVenue)

    @classmethod
    def _editMachine(self, idMachine, hostname, ip, cpu, memory, disk, os, idVenue):
        "Edit a machine"
        MicalaCommunication.editMachine(idMachine, hostname, ip, cpu, memory, disk, os, idVenue)

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()

        if params is not None and params.has_key("action"):
            # Check user input
            hostname = None
            if params.has_key("Hostname"):
                hostname = params["Hostname"].value
            ip = None
            if params.has_key("IP"):
                ip = params["IP"].value
            cpu = None
            if params.has_key("CPU"):
                cpu = params["CPU"].value
            memory = None
            if params.has_key("Memory"):
                memory = params["Memory"].value
                if not memory.isdigit():
                    memory = 0
            disk = None
            if params.has_key("Disk"):
                disk = params["Disk"].value
                if not disk.isdigit():
                    disk = 0
            os = None
            if params.has_key("OS"):
                os = params["OS"].value
            idVenue = None
            if params.has_key("idVenue"):
                idVenue = params["idVenue"].value

            # insert new machine
            if params["action"].value == "insertMachine":
                self._insertNewMachine(hostname, ip, cpu, memory, disk, os, idVenue)
            # edit a machine
            elif params["action"].value == "editMachine":
                if params.has_key("idMachine") and params["idMachine"].value.isdigit():
                    self._editMachine(params["idMachine"].value, hostname, ip, cpu, memory, disk, os, idVenue)

        # retrieve machine list
        self._getMachineList()

