# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

import os, Cookie

class MicalaMonitorController(MicalaController):
    "Monitor Controller to filter LO list"
    # template file for monitor controller
    templatePage = "monitor"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()

        # filters for the page:
        # lecture age
        # lecture contentType: '-1' or conf value
        # orphanStatus: 0, 1, -1
        # finished: 0, 1, -1
        # sortBY
        # sortAscending
        filterAgeLecture = '1month'
        filterContentType = '-1'
        filterOrphanStatus = -1
        filterFinished = -1
        sortBy = 'DateCreated'
        sortOrder = 'DESC'

        # Cookie check
        cookieExpiration = 365*24*60*60
        stringCookie = os.environ.get('HTTP_COOKIE', '')
        # SmartCookie can store dictionary, but it's deprecated because of security hole
        cookieVar = Cookie.SimpleCookie()
        cookieVar.load(stringCookie)
        # Read from cookie default values
        if cookieVar.get('filterAgeLecture'):
            filterAgeLecture = cookieVar['filterAgeLecture'].value
        if cookieVar.get('filterContentType'):
            filterContentType = cookieVar['filterContentType'].value
        if cookieVar.get('filterOrphanStatus'):
            filterOrphanStatus = int(cookieVar['filterOrphanStatus'].value)
        if cookieVar.get('filterFinished'):
            filterFinished = int(cookieVar['filterFinished'].value)
        if cookieVar.get('sortBy'):
            sortBy = cookieVar['sortBy'].value
        if cookieVar.get('sortOrder'):
            if cookieVar['sortOrder'].value == 'DESC':
                sortOrder = cookieVar['sortOrder'].value
            else:
                sortOrder = 'ASC'

        self.pageVars['confContentTypeWebLecture'] = conf.contentTypeWebLecture
        self.pageVars['confContentTypePlainVideo'] = conf.contentTypePlainVideo
        self.pageVars['filterAgeLecture'] = filterAgeLecture
        self.pageVars['filterContentType'] = filterContentType
        self.pageVars['filterOrphanStatus'] = filterOrphanStatus
        self.pageVars['filterFinished'] = filterFinished
        self.pageVars['sortBy'] = sortBy
        self.pageVars['sortOrder'] = sortOrder


