# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

import os, Cookie

class MicalaMonitorListController(MicalaController):
    "MonitorList Controller to view and monitor all the lecture objects"
    # template file for monitor list controller
    templatePage = "monitorList"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getLectureObjectList(self, age, contentType, orphanStatus, finished, sortBy, sortOrder):
        "Retrieve all LO and show it, filtered with params"
        listTasks = MicalaCommunication.getTaskList()
        if contentType == '-1':
            contentType = ""

        (listLectures, listStatuses) = MicalaCommunication.getLectureAndStatusLists("*",
                                                                                    ageLecture  = age,
                                                                                    contentType = contentType,
                                                                                    orphan      = orphanStatus,
                                                                                    sortBy      = sortBy,
                                                                                    sortOrder   = sortOrder)

        self.pageVars['confContentTypeWebLecture'] = conf.contentTypeWebLecture
        self.pageVars['confContentTypePlainVideo'] = conf.contentTypePlainVideo
        self.pageVars['listLectures'] = listLectures
        self.pageVars['listStatuses'] = listStatuses
        self.pageVars['listTasks'] = listTasks
        self.pageVars['filterFinished'] = finished

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()
        self.pageVars['listLectures'] = ''
        self.pageVars['confContentTypeWebLecture'] = ''
        self.pageVars['confContentTypePlainVideo'] = ''
        self.pageVars['listLectures'] = ''
        self.pageVars['listStatuses'] = ''
        self.pageVars['listTasks'] = ''
        self.pageVars['filterFinished'] = ''

        # filters for the page:
        # lecture age
        # lecture contentType
        # orphanStatus
        # finished
        # sortBY
        # sortAscending
        filterAgeLecture = '1month'
        filterContentType = '-1'
        filterOrphanStatus = -1
        filterFinished = -1
        sortBy = 'DateCreated'
        sortOrder = 'DESC'

        # Cookie check
        cookieExpiration = 365*24*60*60
        stringCookie = os.environ.get('HTTP_COOKIE', '')
        cookieVar = Cookie.SimpleCookie()
        cookieVar.load(stringCookie)

        if params:
            # request to filter LO list
            if params.has_key('filterAgeLecture'):
               filterAgeLecture = params['filterAgeLecture'].value
            else:
                # show all
                filterAgeLecture = ''
            if params.has_key('filterContentType'):
                filterContentType = params['filterContentType'].value
            if params.has_key('filterOrphanStatus'):
                filterOrphanStatus = int(params['filterOrphanStatus'].value)
            if params.has_key('filterFinished'):
                filterFinished = int(params['filterFinished'].value)
            if params.has_key('sortBy'):
                sortBy = params['sortBy'].value
            if params.has_key('sortOrder'):
                if params['sortOrder'].value == 'DESC':
                    sortOrder = params['sortOrder'].value
                else:
                    sortOrder = 'ASC'

        # save params into cookies or new cookie. SimpleCookie can save only strings
        cookieVar['filterAgeLecture'] = str(filterAgeLecture)
        cookieVar['filterAgeLecture']['expires'] = cookieExpiration
        cookieVar['filterContentType'] = str(filterContentType)
        cookieVar['filterContentType']['expires'] = cookieExpiration
        cookieVar['filterOrphanStatus'] = str(filterOrphanStatus)
        cookieVar['filterOrphanStatus']['expires'] = cookieExpiration
        cookieVar['filterFinished'] = str(filterFinished)
        cookieVar['filterFinished']['expires'] = cookieExpiration
        cookieVar['sortBy'] = str(sortBy)
        cookieVar['sortBy']['expires'] = cookieExpiration
        cookieVar['sortOrder'] = str(sortOrder)
        cookieVar['sortOrder']['expires'] = cookieExpiration
        print cookieVar

        # retrieve machine list
        self._getLectureObjectList(filterAgeLecture, filterContentType, filterOrphanStatus, filterFinished, sortBy, sortOrder)

