# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

class MicalaTasksController(MicalaController):
    "Tasks Controller to manage tasks"
    # template file for tasks controller
    templatePage = "tasks"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getTaskList(self):
        "Retrieve taskList and show it"
        listTasks = MicalaCommunication.getTaskList()
        self.pageVars['listTasks'] = listTasks

    @classmethod
    def _insertNewTask(self, newTaskName, taskAfter):
        "Insert a new Task after selected one"
        MicalaCommunication.insertNewTaskAfter(newTaskName, taskAfter)

    @classmethod
    def _editTaskName(self, idTask, changedTaskName):
        "Edit task name"
        MicalaCommunication.editTaskName(idTask, changedTaskName)

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()

        if params is not None and params.has_key("action"):
            # insert a new task after selected
            if params["action"].value=="insertTask" and params["taskAfter"].value.isdigit():
                self._insertNewTask(params["newTaskName"].value, params["taskAfter"].value)
            # edit task name
            elif params["action"].value=="editTaskName" and params["idTask"].value.isdigit():
                self._editTaskName(params["idTask"].value, params["changedTaskName"].value)

        # retrieve tasks list
        self._getTaskList()

