# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

from MicalaController import *
from micala import MicalaCommunication

class MicalaVenuesController(MicalaController):
    "Venues Controller to venues"
    # template file for venues controller
    templatePage = "venues"
    # dictionary to pass to the template
    pageVars = None

    @classmethod
    def _getVenueList(self):
        "Retrieve venueList and show it"
        listVenues = MicalaCommunication.getVenueList()
        self.pageVars['listVenues'] = listVenues

    @classmethod
    def _insertNewVenue(self, roomName, IndicoRoomID):
        "Insert a new venue"
        MicalaCommunication.insertNewVenue(roomName, IndicoRoomID)

    @classmethod
    def _editVenue(self, idVenue, roomName, IndicoRoomID):
        "Edit a venue"
        MicalaCommunication.editVenue(idVenue, roomName, IndicoRoomID)

    @classmethod
    def _process(self, params):
        "Dispatcher action"
        # initialize a dictionary for the variables to pass to the template
        self.pageVars = dict()

        if params is not None and params.has_key("action"):
            # Check user input
            roomName = None
            if params.has_key("RoomName"):
                roomName = params["RoomName"].value
            IndicoRoomID = None
            if params.has_key("IndicoRoomID"):
                IndicoRoomID = params["IndicoRoomID"].value
                if not IndicoRoomID.isdigit():
                    IndicoRoomID = 0

            # insert a new venue
            if params["action"].value == "insertVenue":
                self._insertNewVenue(roomName, IndicoRoomID)
            # edit a venue
            elif params["action"].value == "editVenue":
                if params.has_key("idVenue") and params["idVenue"].value.isdigit():
                    self._editVenue(params["idVenue"].value, roomName, IndicoRoomID)

        # retrieve venue list
        self._getVenueList()

