// ====================================================================
// Copyright (c) 2010 The Regents of the University of Michigan and
// The European Organization for Nuclear Research (CERN). Licensed under the
// Educational Community License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License. You may
// obtain a copy of the License at
//
// http://www.osedu.org/licenses/ECL-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an "AS IS"
// BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing
// permissions and limitations under the License.
// ====================================================================

// On document load, execute this
$(document).ready(function(){
	// Show Start processing button with icon using jQuery UI
	$("#bntStartProcessing").button({
        icons: {
            primary: "ui-icon-video"
        }
	});
	// If start processing button clicked, start processing
	$("#bntStartProcessing").click(function() {
		checkProcessing();
		return false;
    });
	// Show Copy link button
	$("#copyOriginPath").button();
	// If copy link button clicked, copy the origin path (Works only with IE)
	$("#copyOriginPath").click(function() {
		//IE only
		if ( !$.browser.msie )
		{
			$("#errorWorksOnlyIE").dialog({
				resizable: true,
				height: 150,
				width: 400,
				modal: true,
				buttons: {
					"Ok": function() {
						$(this).dialog("close");
					}
				}
			});
		} else
			window.clipboardData.setData('Text', $("#originPath").html());
		return false;
    });
	// Show delete button
	$("#btnDeleteLO").button();
	// If delete button clicked, delete lecture
	$("#btnDeleteLO").click(function() {
		deleteLecture();
		return false;
    });
	// Show tasks status list for this lecture
	updateTasksStatus();
	// Show to-do list for this lecture
	updateToDoList();
	// Show log list for this lecture
	updateLogList();
	// Show previews
	showLecturePreview()
});

//Call tasks status list controller passing idLecture
function updateTasksStatus(){
	idLecture = $('#idLecture').val();
	$('#loadingTasksStatus').css('margin','50px');
	$('#loadingTasksStatus').show();
	$.post('lectureTasksStatus.py',
			'idLecture=' + idLecture,
			function(response){
				$('#loadingTasksStatus').hide();
				$("#tasksStatus").html(response);
			});
}

// Call to-do list controller passing idLecture
function updateToDoList(){
	idLecture = $('#idLecture').val();
	$('#loadingToDoList').css('margin','50px');
	$('#loadingToDoList').show();
	$.post('lectureToDoList.py',
			'idLecture=' + idLecture,
			function(response){
				$('#loadingToDoList').hide();
				$("#toDoList").html(response);
			});
}

//Call log list list controller passing idLecture
function updateLogList(){
	idLecture = $('#idLecture').val();
	$('#loadingLogList').css('margin','50px');
	$('#loadingLogList').show();
	$.post('lectureLogList.py',
			'idLecture=' + idLecture,
			function(response){
				$('#loadingLogList').hide();
				$("#logList").html(response);
			});
}

// check user input when start processing button is pressed
function checkProcessing(){
	// check if inserted cropping time, otherwise set blank
	if($("#cropStartTime").val() == 'hh:mm:ss')
		$("#cropStartTime").val('');
	if($("#cropEndTime").val() == 'hh:mm:ss')
		$("#cropEndTime").val('');

	// Check again time format
	if ( checkTimeFormat("#cropStartTime") && checkTimeFormat("#cropEndTime") ){
		// Check if Start time is before/equal to End time, comparing times in seconds
		if ( ($("#cropStartTime").val() != '') && ($("#cropEndTime").val() != '') ){
			substr = $("#cropStartTime").val().split(':');
			// convert in seconds
			intStart = formatTime_time2int(substr[0],substr[1],substr[2]);

			substr = $("#cropEndTime").val().split(':');
			// convert in seconds
			intEnd = formatTime_time2int(substr[0],substr[1],substr[2]);

			if (intStart >= intEnd){
				// Start time is before/equal end time, show error using jQuery UI
				$("#errorBoxEndBeforeStart").dialog({
					resizable: true,
					height: 150,
					width: 400,
					modal: true,
					buttons: {
						"Ok": function() {
							$(this).dialog("close");
							$("#cropStartTime").focus();
							return;
						}
					}
				});
			} else
				// Start time before end time, so execute processing task
				askConfirmProcessing();
		} else
			// Crop time not inserted, so execute processing task
			askConfirmProcessing();
	}
}

// Show a confirm box dialog for executing processing task using jQuery UI
function askConfirmProcessing(){
	$("#confirmBox").dialog({
		resizable: true,
		height: 150,
		width: 400,
		modal: true,
		buttons: {
			"Start processing": function() {
				$(this).dialog("close");
				// execute processing task
				sendProcessingRequest();
			},
			Cancel: function() {
				$(this).dialog("close");
			}
		}
	});
}

// execute processing task
function sendProcessingRequest(){
	// Get lecture id
	idLecture = $('#idLecture').val();
	cropBegin = '';
	cropEnd = ''
	if ($("#cropStartTime").val() != ''){
		substr = $("#cropStartTime").val().split(':');
		// convert to seconds
		cropBegin = formatTime_time2int(substr[0],substr[1],substr[2]);
	}
	if ($("#cropEndTime").val() != ''){
		substr = $("#cropEndTime").val().split(':');
		// convert to seconds
		cropEnd = formatTime_time2int(substr[0],substr[1],substr[2]);
	}
	
	// call StartProcessing page to execute the task with AJAX
	$('#loading').show();
	$.post('lectureStartProcessing.py',
			'idLecture=' + idLecture + '&cropBegin=' + cropBegin + '&cropEnd=' + cropEnd,
			function(response){
				$('#loading').hide();
				$("#processingResponse").html(response);
				// Refresh tasks status list for this lecture
				updateTasksStatus();
				// Refresh to-do list for this lecture
				updateToDoList();
				// Refresh log list for this lecture
				updateLogList();

				// Show response or error message using jQuery UI
				$("#processingResponse").dialog({
					resizable: true,
					height: 170,
					width: 400,
					modal: true,
					buttons: {
						"Ok": function() {
							$(this).dialog("close");
						}
					}
				});

			});
}

// Change input style to italic when user start to write crop time
function changeStyle(idField){
	if($(idField).val() == 'hh:mm:ss'){
		$(idField).val('');
		$(idField).css('font-style','normal');
	}
}

// Check if crop time format inserted is correct
function checkTimeFormat(idField){
	var valueField = $(idField).val();
	if( (valueField != 'hh:mm:ss') && (valueField != '') ){
		var correct = false;
		// Split th string using ':'
		var substr = valueField.split(':');
		if(substr.length == 3){
			if ( (!isNaN(substr[0]) && substr[0] >= 0 && substr[0] < 100) && // hours < 100
				  (!isNaN(substr[1]) && substr[1] >= 0 && substr[1] < 60) && // 0 =< minutes < 60
				  (!isNaN(substr[2]) && substr[2] >= 0 && substr[2] < 60) ) // 0 =< seconds < 60
				correct = true;
		}
		if (!correct){
			// Formatting error
			showError();
			return false;
		}
	}
	return true;
}

// Show error dialog using jQuery UI
function showError(){
	$("#errorBoxFormat").dialog({
		resizable: true,
		height: 180,
		width: 300,
		modal: true,
		buttons: {
			"Ok": function() {
				$(this).dialog("close");
			}
		}
	});
}

//Show a confirm box dialog for executing processing task using jQuery UI
function deleteLecture(){
	$("#deleteBox").dialog({
		resizable: true,
		height: 250,
		width: 300,
		modal: true,
		buttons: {
			"Delete the Lecture Object": function() {
				$(this).dialog("close");
				// execute deleting
				$("#deleteLO").submit();
			},
			Cancel: function() {
				$(this).dialog("close");
			}
		}
	});
}

// Show previews
function showLecturePreview(idLecture, LOID){
	idLecture = $('#idLecture').val();
	LOID = $('#LOID').val();
	$.post('lecturePreview.py',
			'idLecture=' + idLecture + '&LOID=' + LOID + '&showMethod=Landscape',
			function(response){
				$('#lecturePreviews_' + idLecture).html(response);
			}
	);
}

// Convert time format from hh mm ss to seconds
function formatTime_time2int(hours, minutes, seconds) {
	hours = parseInt(hours, 10);
	minutes = parseInt(minutes, 10);
	seconds = parseInt(seconds, 10);
	return parseInt((hours * 3600) + (minutes * 60) + seconds, 10);
}

// Convert seconds video duration to the format hh mm ss
function formatTime_int2time(intValue) {
	intValue = parseInt(intValue, 10);
	seconds = intValue % 60;
	hours = parseInt(intValue / 3600, 10);
	minutes = parseInt((intValue - 3600 * hours) / 60, 10);
	result[0] = hours;
	result[1] = minutes;
	result[2] = seconds;
	return result;
}