# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

use strict;

my $FS       = '\\';
my $dir_temp = "E:\\temp";
my $dir_los  = "E:\\LectureObjects";
my $tar      = "C:\\WLAP-software\\utilities\\tar.exe";
my $pscp     = "C:\\WLAP-software\\utilities\\pscp.exe";
my $plink    = "C:\\WLAP-software\\utilities\\plink.exe";
my $media_coder = '"C:\\Program Files\\MediaCoder\\mediacoder.exe"';
my $arg_pw   = "-pw PASSWORD";
my $arg_user_host = "upload\@esmane.physics.lsa.umich.edu";

# get list of all tarballs in published directory on server
my $output = qx "$plink $arg_pw $arg_user_host ls /data2/upload-processing/2published";
my @list_tarballs_published = split('\n', $output);
my $tarball_curr = $list_tarballs_published[0];
my ($lecture_object) = $tarball_curr =~ m!(.*)\.tar!;

# print a list of tarballs found in 2published directory
print "found the following tarballs on server:\n";
foreach my $tarball (@list_tarballs_published) {
    print "tarball: $tarball\n";
}

# if no tarballs found, exit gracefully
if ($lecture_object !~ m!^\d+\-!) {
    print "no lecture objects found on server\n";
    exit(0);
}

# download first tarball found, and delete it from the server
print "downloading $tarball_curr\n";
&syscom ("$pscp $arg_pw ${arg_user_host}:/data2/upload-processing/2published/$tarball_curr $dir_temp");
print "deleting $tarball_curr from server\n";
&syscom ("$plink  $arg_pw $arg_user_host rm -f /data2/upload-processing/2published/$tarball_curr");

# unpack tarball
print "unpacking $tarball_curr\n";
chdir ($dir_temp);
&syscom("$tar xf $tarball_curr");

# move unpacked lecture object to Lecture Objects directory
print "moving $lecture_object into $dir_los\n";
&syscom("move $dir_temp$FS$lecture_object $dir_los");

# remove tarball
print "removing $tarball_curr from $dir_temp\n";
&syscom("del $dir_temp$FS$tarball_curr");

#####################################
#Mitch's Part
#####################################

#create ipod_job.bat in the lo
print "creating ipod_job in lo\n";
open (MYFILE, ">$dir_los\\$lecture_object\\logs\\ipod_job.bat");
print MYFILE "C:\\WLAP-software\\ipod_video.pl --lo=$lecture_object > $dir_los\\$lecture_object\\logs\\ipod.txt\n";
close (MYFILE);

#run that ipod_job.bat
print "running ipod_job.bat\n";
&syscom("$dir_los\\$lecture_object\\logs\\ipod_job.bat");

#uses mediacoder's command line functionality to create an ipod compatible video file
#print "creating ipod compatible .mp4";
#&syscom("$media_coder -start -exit -ext ipod.7z $dir_los\\$lecture_object\\media\\master-ipod-mp42.avi"); 


###############################################
# syscom
#
#
###############################################
sub syscom
{
    my ($command) = @_;

    print "EXECUTING: $command\n";
    system ($command);
}
