# ====================================================================
# Copyright (c) 2010 The Regents of the University of Michigan and
# The European Organization for Nuclear Research (CERN). Licensed under the
# Educational Community License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may
# obtain a copy of the License at
#
# http://www.osedu.org/licenses/ECL-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.
# ====================================================================

# THIS IS MEANT TO EVENTUALLY REPLACE upload_lo.pl
# UNDER HEAVY CONSTRUCTION

from optparse import OptionParser
import MySQLdb
import os
import sys
import shutil
import glob
import conf
from micala import *

# do I need this?
import time

def main(argv):

    # parse command line
    (success, message, flag_batch_mode, lecture_id) = parse_command_line(argv)

    # Check for success parsing command line. If not, show error message and quit.
    if not success:
        Common.pr(0, "can not parse command line: %s\n" % message)
        sys.exit(0)

    if flag_batch_mode:
        Common.pr(1, "No lecture object given, entering batch mode and looking for lectures to be uploaded.\n")

        # get list of lecture objects in upload directory
        if os.path.exists(conf.dir_to_upload):
            listLOsToUpload = glob.glob( os.path.join(conf.dir_to_upload, '*') ):
            
            if len(listLOsToUpload) > 0:
                # select first LO in list to process
                lecture_object = listLOsToUpload[0]

            # if there are no LOs, say so and exit    
            else:
                Common.pr(1, "There are no lecture objects to be uploaded.  Exiting\n")
                sys.exit(0)
        # if the upload directory does not exist, there's a problem.
        else:
            Common.pr(0, "The directory %s does not exist!\n")
            sys.exit(1)


    sys.exit(0)


    dirMaterial  = 'G:\\Services\\MediaArchive\\Video\\Public2\\Conferences\\atlas'
    recordingBox = 'cernstream0001'

    # parse command line
    (success, message, flag_batch_mode, lecture_id) = parse_command_line(argv)

    # Check for success parsing command line. If not, show error message and quit.
    if not success:
        Common.pr(0, "can not parse command line: %s\n" % message)
        sys.exit(0)

    # Set up various variables
    pathCameraFile = os.path.join(dirMaterial, '2010', lecture_id, lecture_id + "_camera.flv")
    pathSlidesFile = os.path.join(dirMaterial, '2010', lecture_id, lecture_id + "_slides.flv")
    (timeBegin, LOID) = generateLOID(pathCameraFile, recordingBox)
    Common.pr(1, "Generated LOID = %s\n" % LOID)
    outPath = os.path.join(conf.dir_temp, LOID)
    outPathSlides = os.path.join(outPath, "slides-capture")
    outPathMedia  = os.path.join(outPath, "media")

    if flag_batch_mode:
        # GET IndicoID or LOID FROM MICALA DATABASE.
        pass

    # clean up existing lecture object directory in temp dir
    # (might be left over if this script aborted prematurely)
    if os.path.exists(outPath):
        shutil.rmtree(outPath)

    # create full path for outPath directory
    if not os.path.exists(outPath):
        Common.pr(1, "Creating directory: %s\n" % outPath)
        os.makedirs(outPath)
    
    # copy camera and slide files to lecture object in temp dir
    if not os.path.exists(outPathMedia):
        Common.pr(1, "Creating directory: %s\n" % outPathMedia)
        os.makedirs(outPathMedia)
    Common.pr(1, "Copying camera and slide files.\n")
    shutil.copy(pathCameraFile, os.path.join(outPathMedia, os.path.basename(pathCameraFile)))
    shutil.copy(pathSlidesFile, os.path.join(outPathMedia, os.path.basename(pathSlidesFile)))

    # create full path for outPathSlides directory
    if not os.path.exists(outPathSlides):
        Common.pr(1, "Creating directory: %s\n" % outPathSlides)
        os.makedirs(outPathSlides)

    # set up slide creation command
    # this will export slide video as JPEGs, one frame per second
    command = """%s -i %s -r 1 -an "%s" """ \
        % (conf.ffmpeg, 
           os.path.join(dirMaterial, '2010', lecture_id, "%s_slides.flv" % lecture_id), 
           os.path.join(outPathSlides, "%08d.jpg"))
  
    Common.pr(1, "Generating slide JPEGs (one frame per second)\n")
    Common.pr(1, "running command: " + command + "\n")
    # Spawn a subprocess to run the ffmpeg command and capture output
    proc = subprocess.Popen(command, 
                            shell  = True,
                            stdin  = subprocess.PIPE,
                            stdout = subprocess.PIPE,
                            stderr = subprocess.PIPE,)

    keep_going = True
    while (keep_going):
        text_block = proc.stderr.read(1)
        if not text_block: keep_going = False
#        print text_block,

    # rename slides appropriately, so that they have time stamps starting with lecture start time
    Common.pr(1, "Renaming slides\n")
    for fileSlide in glob.glob( os.path.join(outPathSlides, '*.jpg') ):
        os.rename(fileSlide,
                  os.path.join(os.path.dirname(fileSlide), generateSlideName(timeBegin, os.path.basename(fileSlide))))

    # Convert flv file to master.avi to fit with existing processing (this should be fixed so we don't transcode it twice)
    command = "%s -i \"%s\" -vcodec %s \"%s\"" \
        % (conf.ffmpeg,
           os.path.join(outPathMedia, os.path.basename(pathCameraFile)),
           'msmpeg4v2',
           os.path.join(outPathMedia, "master.avi"))

    Common.pr(1, "Transcoding camera flv file to master.avi.\n")
    Common.pr(1, "running command: " + command + "\n")
    # Spawn a subprocess to run the ffmpeg command and capture output
    proc2 = subprocess.Popen(command, 
                            shell  = True,
                            stdin  = subprocess.PIPE,
                            stdout = subprocess.PIPE,
                            stderr = subprocess.PIPE,)

    keep_going = True
    while (keep_going):
        text_block = proc2.stderr.read(1)
        if not text_block: keep_going = False
#        print text_block,


    # Recursively move entire lecture object from temp fir to upload directory.
    shutil.move(outPath, os.path.join(conf.dir_server_uploads_abs, LOID))

    # Finally, add LOID to database.
    MicalaCommunication.createNewMicalaLecture(LOID, conf.contentTypeWebLecture)

def parse_command_line(arg_list):
    """Parse command line arguments and do basic sanity check."""

    success         = True
    message         = ''
    lecture_object  = ''
    flag_batch_mode = False
    conferenceID    = 0
    contributionID  = 0

    # Create instance of parser
    parser = OptionParser()

    # verbosity level
    parser.add_option("-v", "--verbosity", "--verbose", 
                      dest="verbosity",
                      type="int",
                      action="store", 
                      help="set verbosity level", 
                      metavar="VERBOSITY_LEVEL")

    # debugging level
    parser.add_option("-d", "--debug", 
                      dest="debug",
                      action="store", 
                      help="set debugging level", 
                      metavar="DEBUG_LEVEL")

    # lecture object ID, if user wants to publish a particular lecture
    parser.add_option("--lo", "--lecture", "--lecture_object", 
                      dest="lecture_object",
                      action="store", 
                      help="lecture object ID",
                      metavar="LOID")

    # Parse the command line
    (options, args) = parser.parse_args(arg_list)

    # A lecture object must be specified
    if (options.lecture_object == ''):
        success = False
        flag_batch_mode = True
        message += "No lecture object specified, entering batch mode.\n"
        message += "This feature not yet enabled. Please specify a lecture object on the command line.\n"

    # check flags for validity
    if options.verbosity is not None:
        conf.VERBOSITY_LEVEL = options.verbosity
        if not (int(options.verbosity) >= 0 and int(options.verbosity) <= 10):
            success = False
            message = "verbosity level must be between 0 and 10"

    # check flags for validity
    if options.debug is not None:
        conf.DEBUG_LEVEL     = options.debug
        if not (int(options.debug) >= 0 and int(options.debug) <= 10):
            success = False
            message = "debugging level must be between 0 and 10"

    return (success,
            message,
            flag_batch_mode, 
            options.lecture_object)

# Run as a script
if __name__ == "__main__":
    main(sys.argv[1:])
