/* ====================================================================
 * Copyright (c) 2010 The Regents of the University of Michigan and
 * The European Organization for Nuclear Research (CERN). Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
  ==================================================================== */

// Written by: Jim Irrer irrer@umich.edu jimirrer@gmail.com

#define VC_LIBRARY

#include "canonVC_C50i.h"
#include "canonVC_C50i.c"

// --------------------------------------------------------

// main for testing and debugging.
int main(int argc, char **argv)
{
    VC vc;
    double panRatio;
    double tiltRatio;
    int code;
    char devName[100];
    strcpy(devName, "/dev/ttyS0");

    if (argc > 1) {
        if (strlen(argv[1]) == 1) {
            sprintf(devName, "/dev/ttyS%c", argv[1][0]);;
        }
        else {
            strcpy(devName, argv[1]);
        }
    }
    printf("Using device: %s\n", devName);

#if 0
    char chmod[200];
    sprintf(chmod, "sudo chmod 666 %s", devName);
    system(chmod);
#endif

    int constructCode = VCconstruct(&vc, devName);

    while (1) {
        int response;
        char input[1000];
        unsigned int focus;
        unsigned int zoom;
        int iris;

        VCgetFocusPosition(&vc, &focus);
        VCgetZoomPosition(&vc, &zoom);
        VCgetPosition(&vc);
        VCgetFocusRange(&vc, &(vc.focusMinRange), &(vc.focusMaxRange));
        printf("zoom: %4d   focus: %4d   pan: %7.4lf  tilt: %7.4lf\n",
            zoom, focus, vc.panPosition, vc.tiltPosition);

        printf("\n\nEnter z)oom f)ocus i)ris a)uto m)anual p)an t)ilt P)anspeed T)iltspeed F)ocus mode c)ut s)hutter h)ome q)uit: ");
        // scanf("%c", &response);
        // response = getc(stdin);
        fgets(input, sizeof(input)-1, stdin);
        switch(*input) {
            case 'z':
                printf("Enter zoom %d - %d : ", vc.zoomMinRange, vc.zoomMaxRange);
                fgets(input, sizeof(input)-1, stdin);
                sscanf(input, "%d", &zoom);
                printf("value: %d\n", zoom);
                if (VCsetZoom(&vc, zoom) != 0) {
                    VC_SHOW_ERROR(&vc, "VCsetZoom");
                }
                break;

            case 'f':
                printf("Enter focus %d - %d : ", vc.focusMinRange, vc.focusMaxRange);
                fgets(input, sizeof(input)-1, stdin);
                sscanf(input, "%d\n", &focus);
                printf("value: %d\n", focus);
                if (VCsetFocusPosition(&vc, focus) != 0) {
                    VC_SHOW_ERROR(&vc, "VCsetFocusPosition");
                }
                break;

            case 'i':
                {
                    int lo;
                    int hi;
                    char irisText[10];
                    sscanf(VC_IRIS_F16_0, "%X", &lo);
                    sscanf(VC_IRIS_F1_6, "%X", &hi);
                    printf("Enter iris %d - %d : ", lo, hi);
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%d", &iris);
                    printf("value: %d\n", iris);
                    sprintf(irisText, "%02X", iris);
                    printf("irisText: %s\n", irisText);
                    if (VCsetIris(&vc, irisText) != 0) {
                        VC_SHOW_ERROR(&vc, "VCsetIris");
                    }
                }
                break;

            case 'a':
                if (VCsetFocusMode(&vc, VC_FOCUS_MODE_AUTOMATIC) == 0) {
                    printf("Automatic focus mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetFocusMode");
                }

                if (VCsetAELockMode(&vc, VC_AE_LOCK_ON) == 0) {
                    printf("Lock on AE mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetAELockMode");
                }

                if (VCsetExposureMode(&vc, VC_EXPOSURE_AUTOMATIC) == 0) {
                    printf("Automatic exposure mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetExposureMode");
                }
                break;

            case 'm':
                if (VCsetFocusMode(&vc, VC_FOCUS_MODE_MANUAL) == 0) {
                    printf("Manual focus mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetFocusMode");
                }

                if (VCsetAELockMode(&vc, VC_AE_LOCK_OFF) == 0) {
                    printf("Lock off AE mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetAELockMode");
                }

                if (VCsetExposureMode(&vc, VC_EXPOSURE_MANUAL) == 0) {
                    printf("Manual exposure mode\n");
                }
                else {
                    VC_SHOW_ERROR(&vc, "VCsetExposureMode");
                }
                break;

            case 'F':
                    printf("Enter focus mode a)utomatic m)anual n)ear f)ar : ");
                    fgets(input, sizeof(input)-1, stdin);
                    switch (input[0]) {
                        case 'a' :
                            if (VCsetFocusMode(&vc, VC_FOCUS_MODE_AUTOMATIC) != 0) {
                                VC_SHOW_ERROR(&vc, "VCsetFocusMode VC_FOCUS_MODE_AUTOMATIC");
                            }
                        break;

                        case 'm' :
                            if (VCsetFocusMode(&vc, VC_FOCUS_MODE_MANUAL) != 0) {
                                VC_SHOW_ERROR(&vc, "VCsetFocusMode VC_FOCUS_MODE_MANUAL");
                            }
                        break;

                        case 'n' :
                            if (VCsetFocusMode(&vc, VC_FOCUS_MODE_NEAR) != 0) {
                                VC_SHOW_ERROR(&vc, "VCsetFocusMode VC_FOCUS_MODE_NEAR");
                            }
                        break;

                        case 'f' :
                            if (VCsetFocusMode(&vc, VC_FOCUS_MODE_FAR) != 0) {
                                VC_SHOW_ERROR(&vc, "VCsetFocusMode VC_FOCUS_MODE_FAR");
                            }
                        break;

                        default:
                            printf("No action taken.\n");
                        break;
                    }
                break;

            case 'p':
                {
                    printf("Enter pan angle : ");
                    double angle;
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%lf", &angle);
                    printf("angle: %7.3lf\n", angle);
                    VCgetPosition(&vc);
                    if (VCsetPosition(&vc, angle, vc.tiltPosition) != 0) {
                        VC_SHOW_ERROR(&vc, "VCsetPosition");
                    }
                    VCsleep(0.2);
                    while (VCisAtPosition(&vc, angle, vc.tiltPosition) != 0) {
                        printf("Waiting for camera movement %7.3lf\n", vc.panPosition);
                        VCsleep(0.2);
                    }
                }
                break;

            case 't':
                {
                    printf("Enter tilt angle : ");
                    double angle;
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%lf", &angle);
                    printf("angle: %7.3lf\n", angle);
                    VCgetPosition(&vc);
                    if (VCsetPosition(&vc, vc.panPosition, angle) != 0) {
                        VC_SHOW_ERROR(&vc, "VCsetPosition");
                    }
                    VCsleep(0.2);
                    while (VCisAtPosition(&vc, vc.panPosition, angle) != 0) {
                        printf("Waiting for camera movement %7.3lf\n", vc.tiltPosition);
                        VCsleep(0.2);
                    }
                }
                break;

            case 'P':
                {
                    printf("Enter pan speed : ");
                    double speed;
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%lf", &speed);
                    printf("speed: %7.3lf\n", speed);
                    VCsetPanSpeed(&vc, speed);
                }
                break;

            case 'T':
                {
                    printf("Enter tilt speed : ");
                    double speed;
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%lf", &speed);
                    printf("speed: %7.3lf\n", speed);
                    VCsetTiltSpeed(&vc, speed);
                }
                break;

            case 'c':
                {
                    printf("Enter n for on or f for off : ");
                    fgets(input, sizeof(input)-1, stdin);
                    if (input[0] == 'n') {
                        VCirCut(&vc, VC_IR_CUT_ON);
                        break;
                    }
                    if (input[0] == 'f') {
                        VCirCut(&vc, VC_IR_CUT_OFF);
                        break;
                    }
                    printf("No action taken.\n");
                }
                break;

            case 's':
                {
                    printf("Enter shutter speed (-3 - 27) : ");
                    int speed;
                    fgets(input, sizeof(input)-1, stdin);
                    sscanf(input, "%i", &speed);
                    printf("speed: %2d : 1/%d sec\n", speed, VCconvertShutterSpeed(speed));
                    VCsetShutterSpeed(&vc, speed);
                }
                break;

            case 'r':
                VCreset(&vc);
                break;

            case 'h':
                VChome(&vc);
                break;

            case 'q':
                exit(0);

            case 10:
                break;

            default:
                printf("Unknown command %c\n", *input);
                break;
        }
    }
}
